"""utils.server
Helper utilities to host an A2A agent using the SDK server stack.

Minimal helpers to create a Starlette app with `DefaultRequestHandler` and run Uvicorn.
"""

from __future__ import annotations

import uvicorn  # type: ignore

from a2a.server.apps.jsonrpc.starlette_app import A2AStarletteApplication
from a2a.server.request_handlers.default_request_handler import (
    DefaultRequestHandler,
)
from a2a.server.tasks.inmemory_task_store import InMemoryTaskStore
from a2a.types import AgentCard


def build_starlette_app(
    agent_card,
    *,
    executor,
):
    """Create and return a ready-to-serve Starlette ASGI application.

    Args:
        agent_card: Either a raw ``dict`` or an :class:`a2a.types.AgentCard`
            instance describing the agent.
        executor: Concrete implementation of the A2A ``AgentExecutor``
            interface that will handle incoming requests.

    Returns:
        Starlette: Configured Starlette application with the SDK request handler
        mounted at ``/a2a/v1``.
    """
    # Ensure we have a Pydantic AgentCard instance.
    if isinstance(agent_card, dict):
        agent_card = AgentCard.model_validate(agent_card)  # type: ignore[arg-type]

    # Validate executor ---------------------------------------------------------
    if executor is None:
        raise ValueError('executor must be supplied')

    handler = DefaultRequestHandler(executor, InMemoryTaskStore())
    return A2AStarletteApplication(
        agent_card=agent_card, http_handler=handler
    ).build(rpc_url='/a2a/v1')


def run_agent_blocking(
    name: str,
    port: int,
    agent_card,
    *,
    executor,
) -> None:
    """Spin up a Uvicorn server for the given agent and block the current thread.

    Args:
        name: Human-readable agent name printed on start-up.
        port: TCP port to bind to.
        agent_card: Metadata describing the agent (``dict`` or ``AgentCard``).
        executor: Instance of an ``AgentExecutor`` to handle requests.
    """
    app = build_starlette_app(agent_card, executor=executor)
    print(f'{name} listening on http://localhost:{port}')
    uvicorn.run(app, host='127.0.0.1', port=port, log_level='error')



import context

from a2a_agentspeak.build_server import build_and_run

if __name__ == "__main__":

    host = "127.0.0.1"
    port = 9999

    name = "../../sample_agents/robots/robot"

    build_and_run(name, host, port)
    print("First agent running.")

    print("Going to run a second server on the same address (should report a failure).")
    if build_and_run(name, host, port):
        print("TEST KO")
    else:
        print("TEST OK")



import context

from a2a_agentspeak import build_server, actions


if __name__ == "__main__":

    host = "127.0.0.1"
    port = context.manager_port
    name = "manager"

    build_server.build_and_run(name, host, port)



import context

from a2a_agentspeak import build_server

from sample_agents.requirement_generators import openai_requirement_actions


if __name__ == "__main__":

    host = "127.0.0.1"
    port = 9992

    name = "../../sample_agents/requirement_generators/llm_based_requirement_manager"

    build_server.build_and_run(
        name,
        host,
        port,
        [
            openai_requirement_actions.action_prompt_completeness,
            openai_requirement_actions.action_prompt_generation,
        ],
    )

    # Below : to prevent the process to finish while some threads are running.
    input("Press RETURN to finish")



import os
import sys

from a2a_acl.utils.url import build_url

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))

solution_agent_urls = [
    "http://127.0.0.1:9990/",  # robot
    "http://127.0.0.1:9992/",  # opeanai requirement manager
    "http://127.0.0.1:9993/",  # bad requirement manager
    "http://127.0.0.1:9991/",  # mistral requirement manager
    "http://127.0.0.1:9995/",  # naive requirement manager
    "http://127.0.0.1:9996/",  # stub requirement manager
]

orchestrator_agent_url = "http://127.0.0.1:9980/"

spec = "A function to compare two words."

repository_url = build_url("127.0.0.1", 9970)



import agentspeak
from a2a_acl.protocol_extension.acl_message import Illocution


def check_achievement(functor: str, ast: agentspeak.parser.AstAgent) -> bool:
    """The literal occurs as trigger in a plan."""
    # exists
    for p in ast.plans:
        if (
            p.event.goal_type == agentspeak.GoalType.achievement
            and p.event.head.functor == functor
        ):
            return True
    return False


def check_input_belief(functor: str, ast: agentspeak.parser.AstAgent) -> bool:
    """The literal occurs as trigger in a plan."""
    # exists
    for p in ast.plans:
        if (
            p.event.goal_type == agentspeak.GoalType.belief
            and p.event.head.functor == functor
        ):
            return True
    return False


def check_ask_belief(functor: str, ast: agentspeak.parser.AstAgent) -> bool:
    """The literal occurs in a belief or as a target belief in a plan."""
    # exists
    for b in ast.beliefs:
        if b.functor == functor:
            return True
    # not found in beliefs, check in input beliefs
    return check_input_belief(functor, ast)


tell_illoc = agentspeak.Literal(Illocution.TELL)
achieve_illoc = agentspeak.Literal(Illocution.ACHIEVE)
ask_illoc = agentspeak.Literal(Illocution.ASK)


def check_outgoing_illoc(illoc: agentspeak.Literal) -> bool:
    """Check that this literal represents a valid illocution."""
    return illoc.functor in [Illocution.TELL, Illocution.ACHIEVE, Illocution.ASK, Illocution.PROPOSE, Illocution.UPLOAD]



import context

import argparse

from a2a_agentspeak import build_server

if __name__ == "__main__":

    host = "127.0.0.1"

    parser = argparse.ArgumentParser()
    parser.add_argument('filename')
    parser.add_argument('port')
    args = parser.parse_args()

    build_server.build_and_run(args.filename, host, int(args.port))



from a2a_agentspeak import build_server
from a2a_acl.protocol_extension.send_acl_message import spawn_send_acl_message
from a2a_acl.utils.url import build_url

from a2a_agentspeak.content_codecs.common import python_agentspeak_codec_id

if __name__ == "__main__":

    host = "127.0.0.1"
    port = 9999
    name = "../../sample_agents/robots/robot"
    url = build_url(host, port)

    build_server.build_and_run(name, host, port, [])

    spawn_send_acl_message(url, "achieve", "do_jump", "", python_agentspeak_codec_id)



import context

from a2a_agentspeak import build_server

import agentspeak

failure = agentspeak.Literal("failure")


if __name__ == "__main__":

    host = "127.0.0.1"
    port = 9980

    name = "agent_selector"

    build_server.build_and_run(name, host, port)



import context



from a2a_acl.protocol_extension.send_acl_message import spawn_send_acl_message
from a2a_acl.a2a_utils.card_holder import download_card
from a2a_agentspeak.content_codecs.common import python_agentspeak_codec_id

from a2a_acl.interface.interface import ACLAgentCard
from a2a_acl.utils.strings import neutralize_str
from a2a_acl.utils.url import build_url

host = "127.0.0.1"
port = context.port_client

my_card = ACLAgentCard("Client Agent", "A client agent", [], [])


async def main() -> None:

    sender_agent_url = build_url(host, context.port_sender)
    receiver_agent_url = build_url(host, context.port_receiver)
    my_url = build_url(host, context.port_client)

    try:
        agent_card_to_use = await download_card(sender_agent_url)
        spawn_send_acl_message(
            agent_card_to_use,
            "achieve",
            "do_ping(" + neutralize_str(receiver_agent_url) + ")",
            my_url,
            python_agentspeak_codec_id,
        )
    except Exception as e:
        print("Client failed to fetch the public agent card. Cannot continue.")
        exit(-1)


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



import context

from a2a_agentspeak import build_server


if __name__ == "__main__":

    host = "127.0.0.1"
    port = 9996

    name = "../../sample_agents/requirement_generators/list/stub_requirement_manager"

    build_server.build_and_run(name, host, port)



import os
import sys

from a2a_acl.utils.url import build_url

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))

advisor_port = 9989
manager_port = 9981
validator_port = 9999

specification = "A function that says hello."
code = 'def hello(name): print("hello " + name + ".")'




import context

from a2a_agentspeak import build_server

if __name__ == "__main__":

    port = context.acl_agent_port
    host = context.host
    name = "../../sample_agents/robots/robot"

    build_server.build_and_run(name, host, port)



import context

from a2a_agentspeak.build_server import build_and_run

host = "127.0.0.1"
port = context.port_sender

name = "sender"

if __name__ == "__main__":
        build_and_run(name, host, port)




import io

import agentspeak

from agentspeak import LinkedList
from agentspeak.lexer import TokenStream
from agentspeak.parser import AstLiteral
from agentspeak.runtime import BuildTermVisitor

from a2a_acl.protocol_extension.acl_message import ACLMessage, Illocution, ACLIncomingMessage


def lit_of_str(s: str) -> agentspeak.Literal:
    """Use the python-agentspeak parser."""
    # see agentspeak/parser.py
    log = agentspeak.Log(agentspeak.get_logger(__name__), 3)
    fake_source_file = io.StringIO(s + " .")
    fake_source_file.name = "inline"
    tokens = TokenStream(fake_source_file, log, 1)
    tok = next(tokens)
    (_, lit) = agentspeak.parser.parse_literal(tok, tokens, log)
    assert isinstance(lit, AstLiteral)
    return lit.accept(BuildTermVisitor({}))


# deprecated
def parse_asp_linked_list(s: str):
    # FIXME : use a grammar instead to allow nested lists
    if s == "()":
        return ()  # empty Linked list implemented by empty tuple
    else:
        assert s.startswith("[") and s.endswith("]")
        inside = s.removeprefix("[").removesuffix("]")
        if inside == "":
            return ()
        else:
            (head, tail) = inside.split("|", 1)
            return LinkedList(lit_of_str(head), parse_asp_linked_list(tail))


def strplan(p: str):
    return agentspeak.Literal("plain_text", (p,))


def add_source(lit: agentspeak.Literal, s: str) -> agentspeak.Literal:
    return lit.with_annotation(agentspeak.Literal("source", (agentspeak.Literal(s),)))


def add_task_id(lit: agentspeak.Literal, s: str | None) -> agentspeak.Literal:
    if s is None:
        return lit
    else:
        return lit.with_annotation(
            agentspeak.Literal("task_id", (agentspeak.Literal(s),))
        )

def add_message_id(lit: agentspeak.Literal, s: str) -> agentspeak.Literal:
    return lit.with_annotation(agentspeak.Literal("message_id", (s,)))


def goal_type(m: ACLMessage) -> agentspeak.GoalType:
    _i = m.illocution
    if _i == "tell" or _i == "untell":
        return agentspeak.GoalType.belief
    elif _i == "achieve" or _i == "unachieve":
        return agentspeak.GoalType.achievement
    elif _i == "tellHow" or _i == "untellHow":
        return agentspeak.GoalType.tellHow
    else:
        raise RuntimeError("Illocution not supported: " + _i)


def trigger(m: ACLMessage) -> agentspeak.Trigger:
    _i = m.illocution
    if _i == "tell" or _i == "achieve" or _i == "tellHow":
        return agentspeak.Trigger.addition
    elif _i == "untell" or _i == "unachieve" or _i == "untellHow":
        return agentspeak.Trigger.removal
    else:
        raise RuntimeError("Illocution not supported: " + _i)


def build_literal(m: ACLIncomingMessage) -> agentspeak.Literal:
    _i = m.illocution
    _c = m.content
    _s = m.sender
    _mid = m.origin.message.message_id
    if _i in [Illocution.TELL, "untell", "achieve", "unachieve", Illocution.ASK]:
        lit = lit_of_str(_c).freeze({}, {})

    elif _i in ["tellHow", "untellHow"]:
        lit = strplan(_c).freeze({}, {})
    elif _i in [Illocution.UPLOAD, Illocution.FAILURE]:
        # content is not parsed
        lit = agentspeak.Literal(_i, (_c,))
    else:
        raise RuntimeError("Illocution not supported: " + _i)
    return add_message_id(add_task_id(add_source(lit, _s), m.task_id), _mid)



import os

import agentspeak
from agentspeak import AslError
import litellm

from a2a_agentspeak import actions
from a2a_acl.utils.strings import neutralize_str, clear

llm_model = "mistral/mistral-small-latest"

os.environ["LANGFUSE_OTEL_HOST"] = "https://cloud.langfuse.com"
litellm.callbacks = ["langfuse_otel"]


failure = agentspeak.Literal("llm_failure")


def log(m):
    print("[LOG] " + m)



def ask_llm_for_completion(spec: str, req_list: str, rejected_list = None):
    try:
        content = "Specification: " + spec + "(end of the specification)" + " List of requirements: " + req_list + "(end of list of requirements)." + ("" if rejected_list is None else (
                    "(Note that the following requirements do not apply: " + str(rejected_list) + ")"))
        chat_response = litellm.completion(
            model=llm_model,
            messages=[
                {
                    "role": "system",
                    "content": "Given a specification of a system, and a list of atomic requirements, give an atomic requirements that covers the specification and which is not included in the given list of requirements."
                    + " Answer with the new requirement, don't explain.",
                },
                {
                    "role": "user",
                    "content": content,
                },
            ],
            max_tokens=50,
            temperature=1.2,
        )
        res = chat_response.choices[0].message.content
        log(
            "I had an interaction with " + llm_model + " to generate a requirement.\n "
            + "I gave the following info: "
            + content
            +"\n"
            + "The LLM " + llm_model + " gave me the following answer: "
            + res
        )
        return res
    except Exception as e:
        print(str(e))
        raise AslError("LLM failure: " + str(type(e))) from e





def prompt_generation(s, req, rej) -> agentspeak.Literal:
    assert s.functor == "spec"
    assert req.functor == "req"
    assert rej.functor == "rejected_list"
    try:
        res = ask_llm_for_completion(str(s.args[0]), str(req.args[0]), str(rej.args[0]))
        res = clear(res)
        return agentspeak.Literal("generated_req", (res,))
    except Exception as e:
        print("Failure while talking with the llm")
        print(str(e))
        return failure


action_prompt_generate = actions.Action(
    "function",
    ".prompt_generate_alt",
    (agentspeak.Literal, agentspeak.Literal, agentspeak.Literal),
    prompt_generation,
)



import asyncio
from threading import Thread

import agentspeak
import httpx
from a2a.client import A2ACardResolver, A2AClientJSONError, A2AClientHTTPError
from a2a.types import AgentCard

from a2a_acl.interface.card_conversion import bdi_card_from_a2a_card
from a2a_acl.interface.interface import SkillRequest

from a2a_acl.protocol_extension.acl_message import Illocution


def has_declared_skill(
    illoc: Illocution, content: agentspeak.Literal, card: AgentCard
) -> bool:
    """To be used before sending a message."""
    r = SkillRequest(illoc, content.functor, len(content.args))
    c = bdi_card_from_a2a_card(card)
    return c.has_declared_skill(r)


async def check_recipient_intf(
    to_url: str, illoc: Illocution, content: agentspeak.Literal
):
    async with httpx.AsyncClient() as httpx_client:

        resolver = A2ACardResolver(
            httpx_client=httpx_client,
            base_url=to_url,
        )

        try:
            _public_card = await resolver.get_agent_card()
            if not has_declared_skill(illoc, content, _public_card):
                print(
                    "WARNING: the recipient of the message does not publish that skill: "
                    + illoc
                    + " "
                    + str(content.functor)
                    + "/"
                    + str(len(content.args))
                    + "."
                )
        except A2AClientJSONError as e:
            print("---FAIL---: ASL agent failed to send (JSON). " + str(e))
        except A2AClientHTTPError as e:
            print("---FAIL---: ASL agent failed to send (HTTP). " + str(e))
        except Exception as e:
            print("---FAIL---: ASL agent failed to send (other). " + str(e))


class CheckRecipientThread(Thread):
    def __init__(self, to_url: str, illoc: Illocution, content: agentspeak.Literal):
        super().__init__()
        self.to_url = to_url
        self.illoc = illoc
        self.content = content

        def run(self):
            loop = asyncio.new_event_loop()  # loop = asyncio.get_event_loop()
            loop.run_until_complete(
                check_recipient_intf(self.to_url, self.illocution, self.content)
            )
            loop.close()


def spawn_check_recipient_intf(
    to_url: str, illoc: Illocution, content: agentspeak.Literal
) -> None:
    t = CheckRecipientThread(to_url, illoc, content)
    t.start()



import asyncio
import time
from queue import SimpleQueue

import agentspeak
import agentspeak.runtime
import agentspeak.ext_stdlib

from a2a.server.events import EventQueue
from a2a.server.tasks import TaskUpdater
from a2a.types import TaskStatusUpdateEvent, Task, TaskState, TextPart, Part
from a2a.utils import new_task
from a2a_acl import protocol_extension
from a2a_acl.content_codecs.common import natural_language_id
from agentspeak import Literal

import a2a_acl.protocol_extension.acl_message


from a2a_agentspeak import a2a_stdlib
from a2a_agentspeak.asp_message import build_literal, trigger, goal_type, add_message_id

from a2a_acl.agent.acl_agent import ACLAgentExecutor
from a2a_acl.protocol_extension.acl_message import Illocution, ACLIncomingMessage

from a2a_agentspeak.check import check_outgoing_illoc
from a2a_acl.protocol_extension.send_acl_message import (
    default_message_handler, default_artifact_update_handler, build_acl_message, spawn_send_acl_message,
)
from a2a_agentspeak.send_message import (
    spawn_check_recipient_intf,
)
from a2a_agentspeak.content_codecs import python_agentspeak_codec, cold_repository_codec, hot_repository_codec

from a2a_agentspeak.content_codecs.common import (
    python_agentspeak_codec_id,
    UnknownCodec,
    cold_repository_down_codec_id,
    hot_repository_down_codec_id,
    atom_codec_id,
    python_codec_id,
)
from a2a_acl.interface.interface import ACLAgentCard
from a2a_agentspeak.actions import Action

from a2a_acl.a2a_utils.send_message import sync_reply, extract_text_from_message, extract_text_from_task_artifact, build_message

def functor_of_task_state(s:TaskState)->str:
    return s.name

def final_state(t:Task)->bool | None:
    match t.status.state:
        case TaskState.completed | TaskState.failed | TaskState.canceled | TaskState.rejected:
            return True
        case TaskState.submitted | TaskState.working:
            return False
        case TaskState.input_required | TaskState.auth_required :
            return None

def send_error_handler():
    print("Error on send.")


def fast_do_send(
    to_url: str,
    illoc: Illocution,
    content: agentspeak.Literal | str,
    my_url: str,
    codec: str,
    message_handler,
    artifact_handler,
    status_handler ,
    error_handler = send_error_handler # FIXME : not used.
) -> str:
    try:
        spawn_check_recipient_intf(to_url, illoc, content)
    except Exception:
        print("Error: Cannot connect to " + to_url)

    m=spawn_send_acl_message(
        dest=to_url,
        illocution=str(illoc),
        content=str(content),
        sender=my_url,
        codec=codec,
        message_callback=message_handler,
        artifact_update_callback=artifact_handler,
        status_update_callback=status_handler,
        error_callback=error_handler,
    )

    return m.message_id



class EmbeddedASPAgent:
    """Encapsulates a running AgentSpeak agent."""

    def __init__(
        self,
        asl_file: str,
        url: str,
        lib: agentspeak.Actions,
        additional_actions: set[Action],
    ):
        self.queue = SimpleQueue()
        self.is_running = False
        self.current_async_tasks = []
        self.current_updaters = {}
        self.my_url = url

        self.env = agentspeak.runtime.Environment()

        # add custom actions (must occur before loading the asl file)
        self.action_lib = lib
        self.add_custom_actions()
        for t in additional_actions:
            self.add_action(t)

        with open(asl_file) as source:
            self.asp_agent = self.env.build_agent(source, self.action_lib)

    def push_call(self,a,b,c):
        self.queue.put((a,b,c))


    def pull_and_run(self):
        self.asp_agent.run()
        while not self.queue.empty():
            (a,b,c) = self.queue.get()
            self.asp_agent.call(a,b,c, agentspeak.runtime.Intention())
            self.asp_agent.run()
        d = self.asp_agent.shortest_deadline()
        if d is not None:
            time.sleep(d - time.time())
            self.pull_and_run() # Recursion


    def push_and_run(self, a, b ,c):
        self.push_call(a,b,c)
        self.run()

    def run(self):
        if not self.is_running:
            self.is_running = True
            self.pull_and_run()
            self.is_running = False



    def get_updater(self, task_id) -> TaskUpdater:
        return self.current_updaters[task_id]

    def add_updater(self, task_id, updater: TaskUpdater):
        self.current_updaters[task_id] = updater

    def remove_updater(self, task_id):
        self.current_updaters[task_id] = None

    def start(self):
        self.env.run_agent(self.asp_agent)

    # this method is called by __init__
    def add_custom_actions(self):
        actions = self.action_lib

        @actions.add_function(".send_m", (None, agentspeak.Literal, None, None))
        def _fast_send_to_url_m(
            u: agentspeak.Literal, illoc: agentspeak.Literal, term: agentspeak.Literal, callback_atom
        ) -> str:
            assert check_outgoing_illoc(illoc)

            # Use the simplest codec among atom and agentspeak
            codec = atom_codec_id if len(term.args) == 0 else python_agentspeak_codec_id

            id=fast_do_send(
                to_url=str(u),
                illoc=protocol_extension.acl_message.Illocution(str(illoc)),
                content=term,
                my_url=self.my_url,  # Remark : here self is a ACLAgent and not an agentspeak.agent. For this reason it is difficult to get that function out of the class.
                codec=codec,
                message_handler=lambda message: self.process_message_signal(
                    extract_text_from_message(message), None
                ),
                artifact_handler=lambda task: self.process_task_artifact_update(txt=extract_text_from_task_artifact(task), callback_functor=str(callback_atom)),
                status_handler=lambda t, e, i: self.process_task_status_update(t, e, i),
                error_handler=lambda: self.process_signal(None, "send_error", "send error")
            )
            return id

        @actions.add_procedure(".send", (None, agentspeak.Literal, None))
        def _fast_send_to_url(
                u: agentspeak.Literal, illoc: agentspeak.Literal, term: agentspeak.Literal
        ):
            assert check_outgoing_illoc(illoc)

            # Use the simplest codec among atom and agentspeak
            codec = atom_codec_id if len(term.args) == 0 else python_agentspeak_codec_id

            fast_do_send(
                to_url=str(u),
                illoc=protocol_extension.acl_message.Illocution(str(illoc)),
                content=term,
                my_url=self.my_url,
                # Remark : here self is a ACLAgent and not an agentspeak.agent. For this reason it is difficult to get that function out of the class.
                codec=codec,
                message_handler=lambda message: self.process_message_signal(
                    extract_text_from_message(message), None
                ),
                artifact_handler=lambda task: self.process_task_artifact_update(extract_text_from_task_artifact(task), None),
                status_handler=lambda t, e, i: self.process_task_status_update(t, e, i),
                error_handler=lambda: self.process_signal(None, "send_error", "send error")
            )
            return

        @actions.add_procedure(
            ".send_cb", (None, agentspeak.Literal, None, agentspeak.Literal)
        )
        def _fast_send_to_url_cb(
            u: agentspeak.Literal,
            illoc: agentspeak.Literal,
            term: agentspeak.Literal,
            cb: agentspeak.Literal,
        ):
            """Send an ACL message to an agent, specifying a callback for synchronous replies."""
            assert check_outgoing_illoc(illoc)

            # Use the simplest codec among atom and agentspeak
            codec = atom_codec_id if len(term.args) == 0 else python_agentspeak_codec_id

            fast_do_send(
                str(u),
                a2a_acl.protocol_extension.acl_message.Illocution(str(illoc)),
                term,
                self.my_url,
                # Remark : here self is a ACLAgent and not an agentspeak.agent. For this reason it is difficult to get that function out of the class.
                codec,
                message_handler=lambda message: self.process_message_signal(
                    extract_text_from_message(message), cb.functor
                ),
                artifact_handler=lambda task: self.process_task_artifact_update(
                    extract_text_from_task_artifact(task), cb.functor
                ),
                status_handler=lambda t, e, i: self.process_task_status_update(t, e, i),
                error_handler= lambda: self.process_signal(None, "send_error", "send error")
            )

        @actions.add_procedure(".send_int", (None, agentspeak.Literal, int))
        def _fast_send_int_to_url(
            u: agentspeak.Literal, illoc: agentspeak.Literal, term: int
        ):
            assert check_outgoing_illoc(illoc)

            codec = python_codec_id

            fast_do_send(
                str(u),
                protocol_extension.acl_message.Illocution(str(illoc)),
                str(term),
                self.my_url,
                # Remark : here self is a ACLAgent and not an agentspeak.agent. For this reason it is difficult to get that function out of the class.
                codec,
                message_handler=lambda message: self.process_message_signal(
                    extract_text_from_message(message), None
                ),
                artifact_handler=lambda task: self.process_task_artifact_update(extract_text_from_task_artifact(task), None),
                status_handler=lambda t, e, i: self.process_task_status_update(t, e, i),
                error_handler=lambda: self.process_signal(None, "send_error", "send error")
            )

        @actions.add_function(".send_str_cb", (None, agentspeak.Literal, str, agentspeak.Literal))
        def _fast_send_string_to_url_cb(
                u: agentspeak.Literal, illoc: agentspeak.Literal, term: str,
                cb: agentspeak.Literal
        ) -> str:
            assert check_outgoing_illoc(illoc)

            codec = natural_language_id

            id=fast_do_send(
                str(u),
                protocol_extension.acl_message.Illocution(str(illoc)),
                str(term),
                self.my_url,
                # Remark : here self is a ACLAgent and not an agentspeak.agent. For this reason it is difficult to get that function out of the class.
                codec,
                message_handler=lambda message: self.process_message_signal(
                    extract_text_from_message(message), cb.functor
                ),
                artifact_handler=lambda task: self.process_task_artifact_update(extract_text_from_task_artifact(task), cb.functor),
                status_handler=lambda t, e, i: self.process_task_status_update(t, e, i),
                error_handler=lambda: self.process_signal(None, "send_error", "send error")
            )
            return id

        @actions.add_procedure(".send_str", (None, agentspeak.Literal, str))
        def _fast_send_string_to_url(
                u: agentspeak.Literal, illoc: agentspeak.Literal, term: str
        ):
            assert check_outgoing_illoc(illoc)

            codec = natural_language_id

            fast_do_send(
                str(u),
                protocol_extension.acl_message.Illocution(str(illoc)),
                str(term),
                self.my_url,
                # Remark : here self is a ACLAgent and not an agentspeak.agent. For this reason it is difficult to get that function out of the class.
                codec,
                message_handler=default_message_handler,
                artifact_handler=default_artifact_update_handler,
                status_handler=lambda t, e, i: self.process_task_status_update(t, e, i),
                error_handler=lambda: self.process_signal(None, "send_error", "send error")
            )

        @actions.add_procedure(
            ".send_int_cb", (None, agentspeak.Literal, int, agentspeak.Literal)
        )
        def _fast_send_int_to_url_cb(
            u: agentspeak.Literal,
            illoc: agentspeak.Literal,
            term: int,
            cb: agentspeak.Literal,
        ):
            assert check_outgoing_illoc(illoc)

            codec = python_codec_id

            fast_do_send(
                str(u),
                protocol_extension.acl_message.Illocution(str(illoc)),
                str(term),
                self.my_url,
                # Remark : here self is a ACLAgent and not an agentspeak.agent. For this reason it is difficult to get that function out of the class.
                codec,
                message_handler=lambda message: self.process_message_signal(
                    extract_text_from_message(message), cb.functor
                ),
                artifact_handler=lambda task: self.process_task_artifact_update(
                    extract_text_from_task_artifact(task), cb.functor
                ),
                status_handler=lambda t, e, i: self.process_task_status_update(t, e, i),
                error_handler=lambda: self.process_signal(None, "send_error", "send error")
            )

        @actions.add_procedure(
            ".close_task", (None)
        )
        def _close_task(
                id,
        ):
            print("Action: close task")
            t_id = str(id)
            if self.get_updater(t_id) is None :
                return
            else :
                try:
                    python_task=asyncio.create_task(self.get_updater(t_id).complete())
                    python_task.add_done_callback(self.current_async_tasks.remove)
                    self.current_async_tasks.append(python_task)
                except Exception as e:
                    print(str(e))

        @actions.add_procedure(
            ".add_artifact_on_task", (None, None)
        )
        def _add_artifact_on_task(
                id,
                term,
        ):
            print("Action: add artifact on task")
            t_id = str(id)
            if self.get_updater(t_id) is None:
                return
            else:
                try:
                    content = Part(root=TextPart(text=str(term)))
                    python_task = asyncio.create_task(self.get_updater(t_id).add_artifact(parts=[content]))
                    python_task.add_done_callback(self.current_async_tasks.remove)
                    self.current_async_tasks.append(python_task)
                except Exception as e:
                    print(str(e))

        @actions.add_procedure(
            ".reject_task", (None, None)
        )
        def _reject_task(
                id,
                term: agentspeak.Literal,
        ):
            print("Action: reject current task")
            t_id = str(id)
            if self.get_updater(t_id) is None:
                return
            else:
                try:
                    python_task=asyncio.create_task(self.get_updater(t_id).reject(build_message(str(term))))
                    python_task.add_done_callback(self.current_async_tasks.remove)
                    self.current_async_tasks.append(python_task)
                except Exception as e:
                    print(str(e))

        @actions.add_procedure(
            ".input_required", (None, None)
        )
        def _input_required(
                id,
                term: agentspeak.Literal,
        ):
            print("Action: input_required for task")
            t_id = str(id)
            if self.get_updater(t_id) is None:
                return
            else:
                try:
                    python_task = asyncio.create_task(self.get_updater(t_id).requires_input(build_message(str(term))))
                    python_task.add_done_callback(self.current_async_tasks.remove)
                    self.current_async_tasks.append(python_task)
                except Exception as e:
                    print(str(e))

        @actions.add_procedure(
            ".working_on_task", (None)
        )
        def _working_on_task(
                id,
        ):
            print("Action: work on current task")
            t_id = str(id)
            if self.get_updater(t_id) is None:
                return
            else:
                try:
                    #spawn_working_update(self.get_updater(t_id))
                    python_task = asyncio.create_task(self.get_updater(t_id).start_work())
                    python_task.add_done_callback(self.current_async_tasks.remove)
                    self.current_async_tasks.append(python_task)

                except Exception as e:
                    print(str(e))

        @actions.add_procedure(
            ".report_failure_for_task", (None, None)
        )
        def _failed_on_task(
                id, term: agentspeak.Literal
        ):
            t_id = str(id)
            print("Action: report failure for task " + t_id)

            if self.get_updater(t_id) is None:
                return
            else:
                try:
                    python_task = asyncio.create_task(self.get_updater(t_id).failed(build_message(str(term))))
                    python_task.add_done_callback(self.current_async_tasks.remove)
                    self.current_async_tasks.append(python_task)
                except Exception as e:
                    print(str(e))

    def add_action(self, action: Action):
        if action.kind == "function":
            self.action_lib.add_function(
                action.action_name, action.arity, action.implementation
            )
        else:
            print("This kind of tool is not supported yet: " + action.kind)

    async def process_tell_message(self, msg: ACLIncomingMessage):
        """Process tell requests following the AgentSpeak defined behavior."""
        assert msg.illocution == Illocution.TELL
        self.push_and_run(
            agentspeak.Trigger.addition,
            agentspeak.GoalType.belief,
            build_literal(msg)
        )


    async def process_unparsed_message(self, msg: ACLIncomingMessage):
        """Process uploads by building a literal upload(...) handled as a tell message."""
        assert msg.illocution in [Illocution.UPLOAD, Illocution.FAILURE]
        self.push_and_run(
            agentspeak.Trigger.addition,
            agentspeak.GoalType.belief,
            build_literal(msg)
        )




    async def process_achieve_message(self, msg: ACLIncomingMessage):
        """Process achieve requests following the AgentSpeak defined behavior. """
        assert msg.illocution == Illocution.ACHIEVE
        self.push_and_run(
            trigger(msg),
            agentspeak.GoalType.achievement,
            build_literal(msg)
        )



    def process_signal(self, txt: str | None, callback_functor: str | None, kind:str):
        if callback_functor is None:
            print("(Received synchronous " + kind + " reply : " + txt + ")")
        else:
            try:
                args = (txt,) if txt is not None else ()
                lit = Literal(callback_functor, args)
                self.push_and_run(
                    agentspeak.Trigger.addition,
                    agentspeak.GoalType.achievement,
                    lit
                )

            except agentspeak.AslError as e:
                print("Warning: " + callback_functor + " not handled (" + str(e) + ")")

    def process_task_artifact_update(self, txt: str, callback_functor: str | None):
        print("(Received a task artifact update.)")
        if callback_functor is None:
            print("(Received an artifact: " + txt + ")")
        else:
            try:
                args = (txt,) if txt is not None else ()
                lit = Literal(callback_functor, args)
                self.push_and_run(
                    agentspeak.Trigger.addition,
                    agentspeak.GoalType.achievement,
                    lit
                )
                #print("[LOG] Pushed artifact callback " +str(lit))

            except agentspeak.AslError as e:
                print("Warning: " + callback_functor + " not handled (" + str(e) + ")")

    def process_task_status_update(self, t: Task, e: TaskStatusUpdateEvent, message_id : str):
        callback_functor = functor_of_task_state(e.status.state)
        #print("([A2A]Received a task status update: " + callback_functor + ".)")

        try:
            m = t.status.message.parts[0].root.text
            args = (m, )
        except Exception :
            args = ()

        lit = add_message_id(Literal(callback_functor, args), message_id) # FIXME : add source and task_id tags
        self.push_and_run(
            agentspeak.Trigger.addition,
            agentspeak.GoalType.achievement,
            lit
        )


    def process_message_signal(self, txt: str, callback_functor: str | None):
        print("(Received a message update.)")
        self.process_signal(txt, callback_functor, "message")

    async def process_cold_repo_tell_message(self, msg: ACLIncomingMessage):
        assert msg.illocution == "tell" and msg.content.startswith(
            "selected(cold_agent("
        )
        (name, holes) = cold_repository_codec.down_codec_object.decode(msg.content)
        if not holes == []:
            raise NotImplementedError()
        a = Literal("cold_agent", (name, ()))
        self.push_and_run(
            trigger(msg),
            goal_type(msg),
            Literal("selected", (a,))
        )


    async def process_hot_repo_message(self, msg: ACLIncomingMessage):
        await self.process_tell_message(msg)

    def extract_from_beliefs(self, a: str):
        r = self.asp_agent.beliefs[(a, 1)]  # fixme : arity
        assert isinstance(r, set)
        if r == set():
            return None
        else:
            tmp = next(iter(r))
            assert isinstance(tmp, agentspeak.Literal)
            assert tmp.functor == a
            assert isinstance(tmp.args, tuple)
            return tmp.args[0]

    def get_belief(self, s: str) -> str | None:
        r = self.extract_from_beliefs(s)
        if r is not None:
            return str(r)
        else:
            return None

    async def preprocess_tell_message(
        self, m: a2a_acl.protocol_extension.acl_message.ACLIncomingMessage, output_event_queue: EventQueue
    ):
        assert m.illocution == Illocution.TELL
        print("(Received a tell message: " + str(m) + ")")

        if not (m.codec == python_agentspeak_codec_id or m.codec == atom_codec_id):
            print("Warning: unknown codec " + m.codec)

        await sync_reply(output_event_queue, "Tell received.")
        await output_event_queue.close() # close the connection as soon as possible
        await self.process_tell_message(m)


    async def preprocess_achieve_message(
        self,
        m: a2a_acl.protocol_extension.acl_message.ACLIncomingMessage,
        output_event_queue: EventQueue
    ):
        assert m.illocution == Illocution.ACHIEVE
        print("(Received an achieve message: " + str(m) + ")")

        if not (m.codec == python_agentspeak_codec_id or m.codec == atom_codec_id):
            print("Warning: unknown codec " + m.codec)

        # On achieve messages, we create a new task.
        a2a_task = new_task(m.origin.message)
        python_task_1 = asyncio.create_task(output_event_queue.enqueue_event(a2a_task))
        python_task_1.add_done_callback(self.current_async_tasks.remove)
        self.current_async_tasks.append(python_task_1)
        await asyncio.sleep(0)

        updater = TaskUpdater(output_event_queue, a2a_task.id, a2a_task.context_id)

        self.add_updater(a2a_task.id, updater)

        # update to submitted
        python_task_3=asyncio.create_task(updater.submit())
        python_task_3.add_done_callback(self.current_async_tasks.remove)
        self.current_async_tasks.append(python_task_3)

        # launch behavior
        python_task_2 = asyncio.create_task(self.process_achieve_message(m))
        python_task_2.add_done_callback(self.current_async_tasks.remove)
        self.current_async_tasks.append(python_task_2)


        # don't return/close the queue before the task is completed
        # join
        while len(self.current_async_tasks) > 0 or not final_state(a2a_task):
            await asyncio.sleep(0.1)
        self.remove_updater(a2a_task.id)
        print("Achieve: Done.")
        return




    async def preprocess_ask_message(
        self, m: a2a_acl.protocol_extension.acl_message.ACLIncomingMessage, output_event_queue: EventQueue
    ):
        """Reply synchronously with the required information."""
        """in A2A, each received message has an event queue to post responses.
                This is not the case in AgentSpeak.
                Here we add an illocution for requests that need an answer : ask"""
        assert m.illocution == Illocution.ASK  # fixme : also check that the requested belief is public.
        print("(Received an ask message: " + str(m) + ")")

        if not (m.codec == python_agentspeak_codec_id or m.codec == atom_codec_id):
            print("Warning: unknown codec " + m.codec)

        result = self.get_belief(m.content)
        if result is not None:
            await sync_reply(output_event_queue, result)
        else:
            pass  # do not reply
        await output_event_queue.close() # close only after synchronous reply is sent


    async def preprocess_cold_repo_tell_message(
        self, m: a2a_acl.protocol_extension.acl_message.ACLIncomingMessage, output_event_queue: EventQueue
    ):
        assert(m.illocution == Illocution.TELL)
        print("(Received a cold repository message: " + str(m) + ")")

        if not (m.codec == cold_repository_down_codec_id):
            print("Warning: unknown codec " + m.codec)

        await sync_reply(output_event_queue, "Tell received.")
        await self.process_cold_repo_tell_message(m)


    async def preprocess_hot_repo_tell_message(
        self, m: a2a_acl.protocol_extension.acl_message.ACLIncomingMessage, output_event_queue: EventQueue
    ):
        assert (m.illocution == Illocution.TELL)
        print("(Received a hot repository message: " + str(m) + ")")

        if not (m.codec == hot_repository_down_codec_id):
            print("Warning: unknown codec " + m.codec)

        await sync_reply(output_event_queue, "Tell received.")
        await self.process_hot_repo_message(m)


    async def preprocess_unparsed_message(
        self, m: a2a_acl.protocol_extension.acl_message.ACLIncomingMessage, output_event_queue: EventQueue
    ):
        assert m.illocution in [Illocution.UPLOAD, Illocution.FAILURE]

        # codec not checked

        print("(Received an unparsed message.)")
        await sync_reply(output_event_queue, f"Message {m.illocution} received.")
        await output_event_queue.close() # close as soon as possible.
        await self.process_unparsed_message(m)



class ASPAgentExecutor(ACLAgentExecutor):

    def __init__(
        self,
        asl_file: str,
        url: str,
        additional_actions,
        card: ACLAgentCard,
    ):
        super().__init__(card, url)
        self.asl_agent = EmbeddedASPAgent(
            asl_file,
            url,
            lib=agentspeak.Actions(a2a_stdlib.actions),
            additional_actions=additional_actions,
        )
        self.codec_objects["python_agentspeak_codec"] = (
            python_agentspeak_codec.codec_object
        )
        self.codec_objects["cold_repository_down_codec"] = (
            cold_repository_codec.down_codec_object
        )

        self.codec_objects["hot_repository_down_codec"] = (
            hot_repository_codec.codec_object
        )

    def start_behavior(self):
        self.asl_agent.start()


    async def execute_tell(
        self,
        m: ACLIncomingMessage,
        output_event_queue: EventQueue,
    ) -> None:
        # FIXME (match)
        if m.codec == python_agentspeak_codec_id or m.codec == atom_codec_id:
            await self.asl_agent.preprocess_tell_message(m, output_event_queue)
        elif m.codec == cold_repository_down_codec_id:
            await self.asl_agent.preprocess_cold_repo_tell_message(
                m, output_event_queue
            )
        elif m.codec == hot_repository_down_codec_id:
            await self.asl_agent.preprocess_hot_repo_tell_message(m, output_event_queue)
        else:
            raise UnknownCodec

    async def execute_achieve(
        self,
        m: ACLIncomingMessage,
        output_event_queue: EventQueue,
    ) -> None:
        # achieve messages are always in the python_agentspeak_codec (possible misuse however because other codecs are accepted)
        await self.asl_agent.preprocess_achieve_message(m, output_event_queue)

    async def execute_ask(
        self,
        m: ACLIncomingMessage,
        output_event_queue: EventQueue,
    ) -> None:
        # ask messages are always in the python_agentspeak_codec (possible misuse however because other codecs are accepted)
        await self.asl_agent.preprocess_ask_message(m, output_event_queue)

    async def execute_on_upload(
        self,
        m: ACLIncomingMessage,
        output_event_queue: EventQueue,
    ) -> None:
        await self.asl_agent.preprocess_unparsed_message(m, output_event_queue)

    async def execute_on_failure(
        self,
        m: ACLIncomingMessage,
        output_event_queue: EventQueue,
    ) -> None:
        await self.asl_agent.preprocess_unparsed_message(m, output_event_queue)





import context

from a2a_agentspeak import build_server, actions


def do_explode():
    print("TEST OK")
    return 0


my_special_action = actions.Action(
    "function",
    ".special",
    (),
    do_explode,
)

if __name__ == "__main__":

    host = "127.0.0.1"
    port = 9999
    name = "../../sample_agents/robots/open_robot"

    build_server.build_and_run(name, host, port, [my_special_action])



import agentspeak.ext_stdlib
import agentspeak.runtime

actions = agentspeak.Actions(agentspeak.ext_stdlib.actions)


@actions.add("jump", 0)
def _jump(a: agentspeak.runtime.Agent, t, i):
    print("[" + a.name + "] I jump")
    yield


@actions.add_procedure(".print_float", (float,))
def _print_float(a):
    print(str(a))


@actions.add_function(".str_concat", (str, str))
def _str_concat(s1: str, s2: str) -> str:
    return s1 + s2


@actions.add_function(".starts_with", (str, str))
def starts_with(a: str, b: str):
    return a.startswith(b)

@actions.add_predicate(".repr_eq", (None,None))
def repr_eq(a, b):
    return str(a) == str(b)




"""agent_Carol.py
AgentCarol – helper agent that visualises or shuffles Bob's guess history.

Carol receives plain-text JSON payloads from AgentBob and returns either
(1) a human-readable table of the guesses so far or (2) a JSON list with
entries randomly shuffled, depending on the request.  This functionality
is intentionally simple to keep the focus on A2A message flow.
"""

import json
import random
import uuid

from typing import Any

from a2a.server.agent_execution.agent_executor import AgentExecutor
from a2a.server.agent_execution.context import RequestContext
from a2a.server.events.event_queue import EventQueue
from a2a.server.tasks.task_updater import TaskUpdater
from a2a.types import AgentCard, Part, TextPart
from a2a.utils.message import get_message_text
from config import AGENT_CAROL_PORT
from utils import try_parse_json
from utils.game_logic import process_history_payload
from utils.server import run_agent_blocking


# ------------------ Agent card ------------------

carol_skills = [
    {
        'id': 'history_visualiser',
        'name': 'Guess History Visualiser',
        'description': 'Generates a formatted text summary of guess/response history to aid the player.',
        'tags': ['visualisation', 'demo'],
        'inputModes': ['text/plain'],
        'outputModes': ['text/plain'],
        'examples': ['[{"guess": 25, "response": "Go higher"}]'],
    },
    {
        'id': 'history_shuffler',
        'name': 'Guess History Shuffler',
        'description': 'Randomly shuffles the order of guess/response entries in a provided history list and returns JSON.',
        'tags': ['shuffling', 'demo'],
        'inputModes': ['text/plain'],
        'outputModes': ['text/plain'],
        'examples': [
            '{"action": "shuffle", "history": [{"guess": 25, "response": "Go higher"}]}'
        ],
    },
]

carol_card = AgentCard.model_validate(
    {
        'name': 'AgentCarol',
        'description': 'Visualises the history of guesses and hints from AgentAlice in a readable table format.',
        'url': f'http://localhost:{AGENT_CAROL_PORT}/a2a/v1',
        'preferredTransport': 'JSONRPC',
        'protocolVersion': '0.3.0',
        'version': '1.0.0',
        'capabilities': {
            'streaming': False,
            'pushNotifications': False,
            'stateTransitionHistory': False,
        },
        'defaultInputModes': ['text/plain'],
        'defaultOutputModes': ['text/plain'],
        'skills': carol_skills,
    }
)

# ------------------ SDK AgentExecutor implementation ------------------


class HistoryHelperExecutor(AgentExecutor):
    """AgentExecutor implementing Carol's visualise / shuffle functionality."""

    @staticmethod
    def _print_guesses(label: str, history: list[dict[str, Any]]) -> None:
        """Utility: print only the numeric guesses in *history* for debugging."""
        try:
            guesses = [int(item.get('guess', '?')) for item in history]
        except Exception:
            guesses = []
        print(f'[Carol] {label}: {guesses}')

    def __init__(self) -> None:
        # Keep the last history list so we can reshuffle it on follow-up.
        self._last_history: list[dict[str, Any]] = []

    # ------------------------------------------------------------------
    # Internal helper methods
    # ------------------------------------------------------------------

    async def _handle_followup(
        self, context: RequestContext, raw_text: str, event_queue: EventQueue
    ) -> None:
        """Handle a follow-up message that references an existing task."""
        task_id = context.task_id or (
            context.message.reference_task_ids[0]  # type: ignore[index]
            if context.message and context.message.reference_task_ids
            else str(uuid.uuid4())
        )
        updater = TaskUpdater(
            event_queue,
            task_id=task_id,
            context_id=context.context_id or str(uuid.uuid4()),
        )

        if raw_text.lower().startswith('well done'):
            print('[Carol] Received well done – completing task')
            await updater.complete()
            return

        # Any other text → shuffle again and ask for more input
        print('[Carol] Shuffling again and returning list')
        random.shuffle(self._last_history)
        # Debug print before sending back to Bob
        self._print_guesses('Shuffled list', self._last_history)
        response_text = json.dumps(self._last_history)
        await updater.add_artifact([Part(root=TextPart(text=response_text))])
        # Ask for another input and signal that this is the last event for this invocation
        await updater.requires_input(final=True)

    async def _handle_initial(
        self, raw_text: str, context: RequestContext, event_queue: EventQueue
    ) -> None:
        """Handle the first message in a new conversation."""
        response_text = (
            process_history_payload(raw_text) if raw_text else 'Invalid input.'
        )

        # Remember history list if provided so we can shuffle again later
        success, parsed = try_parse_json(raw_text)
        if (
            success
            and isinstance(parsed, dict)
            and parsed.get('action') == 'shuffle'
        ):
            hist = parsed.get('history', [])
            if isinstance(hist, list):
                self._last_history = hist

        task_id = context.task_id or str(uuid.uuid4())
        updater = TaskUpdater(
            event_queue,
            task_id=task_id,
            context_id=context.context_id or str(uuid.uuid4()),
        )
        # Debug print before sending initial response
        try:
            if success and isinstance(parsed, list):
                self._print_guesses('Initial list', parsed)
            else:
                self._print_guesses('Initial list', self._last_history)
        except Exception:
            pass
        await updater.add_artifact([Part(root=TextPart(text=response_text))])
        # Ask Bob for further input
        await updater.requires_input(final=True)

    # ------------------------------------------------------------------
    # AgentExecutor interface
    # ------------------------------------------------------------------

    async def execute(
        self, context: RequestContext, event_queue: EventQueue
    ) -> None:
        """Process an incoming message and dispatch to the appropriate handler."""
        raw_text = get_message_text(context.message) if context.message else ''
        is_followup = bool(
            context.message and context.message.reference_task_ids
        )

        if is_followup:
            await self._handle_followup(context, raw_text, event_queue)
        else:
            await self._handle_initial(raw_text, context, event_queue)

    async def cancel(
        self, context: RequestContext, event_queue: EventQueue
    ) -> None:
        """Cancel the ongoing task on explicit request from a peer agent."""
        if context.task_id:
            print(
                f'[Carol] Task {context.task_id} canceled on request of peer agent'
            )
            updater = TaskUpdater(
                event_queue,
                task_id=context.task_id,
                context_id=context.context_id or str(uuid.uuid4()),
            )
            await updater.cancel()


if __name__ == '__main__':
    run_agent_blocking(
        name='AgentCarol',
        port=AGENT_CAROL_PORT,
        agent_card=carol_card,
        executor=HistoryHelperExecutor(),
    )



from a2a.types import AgentCard


from a2a.server.apps import A2AStarletteApplication
from a2a.server.request_handlers import DefaultRequestHandler
from a2a.server.tasks import InMemoryTaskStore
from a2a_acl.protocol_extension.acl_message import Illocution

from a2a_agentspeak.content_codecs.common import (
    python_agentspeak_codec_id,
    cold_repository_down_codec_id,
    hot_repository_down_codec_id,
    atom_codec_id,
)
from a2a_acl.interface import asi_parser
from a2a_acl.interface.card_conversion import a2a_card_from_bdi_card
from a2a_acl.interface.interface import (
    SkillDeclaration,
    ACLAgentCard,
)
from a2a_agentspeak.embedded_agent import ASPAgentExecutor

import agentspeak

from a2a_agentspeak.check import check_achievement, check_input_belief, check_ask_belief
from a2a_acl.utils.result import Result
from a2a_acl.utils.url import build_url


class ASPAgentBuilder:
    skills: list[SkillDeclaration]

    def __init__(self, name, doc, implementation: str, additional_actions):
        self.skills = []
        self.name = name
        self.doc = doc
        self.implementation_file = implementation
        self.additional_actions = additional_actions
        self.codecs = [
            python_agentspeak_codec_id,
            cold_repository_down_codec_id,
            hot_repository_down_codec_id,
            atom_codec_id,
        ]

    def publish(self, illoc, doc, functor, arity):
        self.skills.append(
            SkillDeclaration(
                doc=doc,
                functor=functor,
                arity=arity,
                declaration_kind=illoc,
            )
        )

    def public_functors(self):
        return [s.functor for s in self.skills]

    def build_bdi_card(self):
        return ACLAgentCard(self.name, self.doc, self.skills, self.codecs)

    def build_a2a_card(self, url) -> AgentCard:
        return a2a_card_from_bdi_card(self.build_bdi_card(), url)

    def build_server(self, host:str, port:int) -> A2AStarletteApplication:
        url = build_url(host, port)
        self.executor = ASPAgentExecutor(
            self.implementation_file,
            url,
            additional_actions=self.additional_actions,
            card=self.build_bdi_card(),
        )

        request_handler = DefaultRequestHandler(
            agent_executor=self.executor,
            task_store=InMemoryTaskStore(),
        )
        app = A2AStarletteApplication(
            agent_card=self.build_a2a_card(url), http_handler=request_handler
        )
        return app

    def start_agent_behavior(self):
        self.executor.start_behavior()

    def check(self) -> Result:
        """Check that the public interface corresponds to actual triggers in implementation.
        More precisely:
         * achievements declared in the interface must have a trigger (we do not consider plans that would be added dynamically with askHow or tellHow).
         * belief literals which can be asked must occur in the implementation (we do not consider beliefs that are perceived during execution and which are not handled by the start implementation)
         * beliefs that are told to that agent must occur too in the start implementation.
        """
        LOGGER = agentspeak.get_logger(__name__)
        with open(self.implementation_file) as source:
            log = agentspeak.Log(LOGGER, 3)
            tokens = agentspeak.lexer.TokenStream(source, log)
            ast_agent: agentspeak.parser.ASTAgent = agentspeak.parser.parse(
                source.name, tokens, log
            )
            log.throw()

        # exists
        for s in self.skills:
            potential_error_suffix = " (" + str(s.declaration_kind) + ") declared but not implemented."
            match s.declaration_kind:
                case Illocution.ACHIEVE:
                    if not check_achievement(s.functor, ast_agent):
                        return Result(False, s.functor + potential_error_suffix)
                case Illocution.TELL:
                    if not check_input_belief(s.functor, ast_agent):
                        return Result(False, s.functor + potential_error_suffix)
                case Illocution.UPLOAD:
                    # upload handled as a new belief
                    if not check_input_belief("upload", ast_agent):
                        return Result(False, s.functor + potential_error_suffix)
                case Illocution.FAILURE:
                    # failure handled as a new belief
                    if not check_input_belief("failure", ast_agent):
                        return Result(False, s.functor + potential_error_suffix)
                case Illocution.ASK:
                    if not check_ask_belief(s.functor, ast_agent):
                        return Result(False, s.functor + potential_error_suffix)

                case other :
                    return Result(False, other + "not recognized")
        return Result(True, None)


class InterfaceError(Exception):
    def __init__(self, token):
        self.token = token


def from_files(impl: str, intf: str = None, actions=()) -> ASPAgentBuilder:
    impl_file = impl + ".asl"
    intf_file = (intf if intf is not None else impl) + ".asi"

    i: ACLAgentCard = asi_parser.read_file(intf_file)

    a: ASPAgentBuilder = ASPAgentBuilder(
        i.name, i.doc, impl_file, additional_actions=actions
    )

    for l in i.skills:
        match l.declaration_kind:
            case Illocution.ASK:
                a.publish(Illocution.ASK, l.doc, l.functor, l.arity)
            case Illocution.TELL:
                a.publish(Illocution.TELL, l.doc, l.functor, l.arity)
            case Illocution.ACHIEVE:
                a.publish(Illocution.ACHIEVE, l.doc, l.functor, l.arity)
            case Illocution.UPLOAD:
                a.publish(Illocution.UPLOAD, l.doc, l.functor, l.arity)
            case  Illocution.FAILURE:
                a.publish(Illocution.FAILURE, l.doc, l.functor, l.arity)

            case _:
                assert False

    r = a.check()
    if not r.success:
        print("Could not construct agent because interface does not match implementation.")
        print("Reason: " + r.reason)
        raise InterfaceError(r.reason)
    else:
        return a






import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))

host = "127.0.0.1"
acl_agent_port = 9999
translator_agent_port = 9998
test_client_port = 9997



from a2a.types import AgentCard

from a2a_acl.interface.interface import SkillRequest
from a2a_acl.interface.card_conversion import bdi_card_from_a2a_card
from a2a_acl.a2a_utils.card_holder import CardHolder
from mistral_selector_prompt import ask_llm_for_agent_selection


class HotRepository:
    """A repository of running agents."""

    urls: set[str]
    cards: CardHolder

    def __init__(self):
        self.urls = set()
        self.card_holder = CardHolder()

    async def register(self, url):
        res = await self.card_holder.retrieve_card_from(url)
        if res:
            self.urls.add(
                url
            )  # append only if the card has been retrieved successfully
            print(url + " successfully registered")
        else:
            print("Error: did not manage to get card from " + url)

    def get_cards_by_skills(self, skills: list[SkillRequest]) -> list[AgentCard]:
        print(
            "Received the request to filter cards with following skills: " + str(skills)
        )
        return [
            card
            for card in self.card_holder.cards
            if bdi_card_from_a2a_card(card).has_declared_skills(skills)
        ]

    def get_top_card_by_skills(self, skills: list[SkillRequest]) -> AgentCard:
        cards = self.get_cards_by_skills(skills)
        if cards == []:
            return None
        selected_card = cards[0]
        selected_score = self.card_holder.get_reputation_by_card(selected_card)
        for c in cards:
            s = self.card_holder.get_reputation_by_card(c)
            if s > selected_score:
                selected_score = s
                selected_card = c
        return selected_card

    def select_by_llm(self, prompt: str, skills: list[SkillRequest]) -> AgentCard:
        cards = self.get_cards_by_skills(skills)
        if not cards == []:
            best_score = self.card_holder.get_reputation_by_card(cards[0])
            for c in cards:
                s = self.card_holder.get_reputation_by_card(c)
                if s > best_score:
                    best_score = s
            worse_score = self.card_holder.get_reputation_by_card(cards[0])
            for c in cards:
                s = self.card_holder.get_reputation_by_card(c)
                if s < worse_score:
                    worse_score = s

            def reputation(c):
                r = self.card_holder.get_reputation_by_card(c)
                if best_score == worse_score:
                    return "unknown"
                elif r == worse_score:
                    return "bad"
                elif r == best_score:
                    return "good"
                else:
                    return "neutral"

            annotated_cards = [(c, reputation(c)) for c in cards]
            i = ask_llm_for_agent_selection(prompt, annotated_cards)
            return cards[i]
        else:
            return None

    def print_state(self):
        print("Registered hot agents:")
        for url in self.urls:
            c = self.card_holder.get_card_by_url(url)
            n = c.name if c else "-"
            if n.endswith("\n"):
                n = n.removesuffix("\n")
            print(
                url
                + " ("
                + n
                + ", reputation= "
                + str(self.card_holder.get_reputation_by_url(url))
                + ", with "
                + str(len(c.skills))
                + " skill(s))"
            )
        print("(end)")

    def degrade(self, url: str):
        self.card_holder.degrade(url)
        print(
            "Degraded reputation of "
            + url
            + " to "
            + str(self.card_holder.get_reputation_by_url(url))
        )



import context

import threading

from a2a_agentspeak.build_server import build_and_run
from a2a_acl.utils.url import build_url

host = "127.0.0.1"
port = context.port_player
my_url = build_url(host, port)

name = "player_agent"


if __name__ == "__main__":

    def start():
        build_and_run(name, host, port, [])

    threading.Thread(target=start).start()

    input("Press ENTER to exit.")



import uuid

from a2a_acl.a2a_utils.send_message import spawn_send_message

import context
from a2a.types import AgentCard, Message, Role, Part, TextPart

from a2a_acl.a2a_utils.card_holder import download_card

from a2a_acl.utils.url import build_url

request1 = "Please move dear robot."
request2 = "Your target is located at position 20."
request3 = "Please move by 3 cm."


async def main() -> None:
    print("A2A client (with no A2A server).")
    host = context.host
    my_port = context.test_client_port

    translator_agent_url = build_url(host, context.translator_agent_port)
    my_url = build_url(host, my_port)

    print("Getting A2A agent card of the translator agent.")
    final_agent_card_to_use: AgentCard | None = None
    try:
        final_agent_card_to_use = await download_card(translator_agent_url)

    except Exception:
        print("Client failed to fetch the public agent card. Cannot continue.")
        exit(-1)

    print("Send three messages to the translator.")

    # First message
    # The translator is not an ACL/A2A agent, just an A2A agent.
    spawn_send_message(final_agent_card_to_use,request1, my_url)

    await asyncio.sleep(3)

    # Second message
    msg2 = Message(role=Role.agent, message_id=uuid.uuid4().hex, parts=[Part(root=TextPart(text=request2))])
    spawn_send_message(final_agent_card_to_use, request2, my_url)

    await asyncio.sleep(3)

    # Third message
    spawn_send_message(final_agent_card_to_use, request3, my_url)


print("End.")


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())

    input("Press Enter to exit.")



from a2a_acl.interface import asi_parser
from a2a_acl.interface.interface import ACLAgentCard, SkillRequest


class ColdRepository:
    """A repository of agents that can be instantiated."""

    store: dict[str, tuple[ACLAgentCard, frozenset]]

    def __init__(self, path: str):
        self.store = {}
        self.path = path

    def add(self, file_name: str, holes=()):
        print(
            "Adding "
            + file_name
            + " to the repository of cold agents (from "
            + self.path
            + ")"
        )
        c = asi_parser.read_file(self.path + file_name + ".asi")
        self.store[file_name] = (c, holes)

    def get_agents_by_skills(
        self, skills: list[SkillRequest]
    ) -> list[tuple[str, frozenset]]:
        print(
            "Received the request to filter cold agents with following skills: "
            + str(skills)
        )
        return [
            (n, h)
            for (n, (c, h)) in self.store.items()
            if c.has_declared_skills(skills)
        ]

    def print_state(self):
        print("Registered cold agents:")
        for a, (c, _) in self.store.items():
            n = c.name if c else "-"
            if n.endswith("\n"):
                n = n.removesuffix("\n")
            print(str(a) + " (" + n + ", with " + str(len(c.skills)) + " skill(s))")
        print("(end)")

    def degrade(self, agent):
        raise NotImplementedError()



from dataclasses import dataclass
from typing import Callable


@dataclass
class Action:
    """Datatype for actions than can be added to AgentSpeak agents."""

    kind: str
    action_name: str
    arity: tuple
    implementation: Callable


def check_action_for_hole(hole, action):
    (hole_name, hole_arity) = hole
    return hole_name == action.action_name and hole_arity == len(action.arity)



import os

import agentspeak
from agentspeak import AslError
from mistralai import Mistral

from a2a_agentspeak import actions
from a2a_acl.utils.strings import neutralize_str

llm_api_key = os.environ["MISTRAL_API_KEY"]
llm_model = "mistral-small-latest"

llm_client = Mistral(api_key=llm_api_key)

failure = agentspeak.Literal("failure")


def log(m):
    print("[LOG] " + m)


def ask_llm_for_coverage(spec, req_list) -> bool:
    try:
        chat_response = llm_client.chat.complete(
            model=llm_model,
            messages=[
                {
                    "role": "system",
                    "content": "Given a specification of a system, and a list of atomic requirements, tell if that list of atomic requirements covers well that specification."
                    + " Answer COMPLETE is the specification is well covered."
                    + " Answer PARTIAL otherwise.",
                },
                {
                    "role": "user",
                    "content": "Specification: "
                    + spec
                    + "(end of the specification) List of requirements: "
                    + req_list
                    + "(end of list of requirements).",
                },
            ],
            max_tokens=3,
        )
        r = chat_response.choices[0].message.content
        log(
            "I had an interaction with mistral to check coverage.\n"
            + "I gave the following spec: "
            + spec
            + "\n"
            + "I also gave the following requirements: "
            + req_list
            + "\n"
            + "Mistral gave me the following answer: "
            + r
        )
        if r.startswith("COMPLETE"):
            return True
        elif r.startswith("PARTIAL"):
            return False
        else:
            raise Exception("Cannot understand LLM answer.")
    except Exception as e:
        print(str(e))
        raise AslError("LLM failure: " + str(type(e)))


def ask_llm_for_completion(spec: str, req_list: str):
    try:
        chat_response = llm_client.chat.complete(
            model=llm_model,
            messages=[
                {
                    "role": "system",
                    "content": "Given a specification of a system, and a list of atomic requirements, give an atomic requirements that covers the specification and which is not included in the given list of requirements."
                    + " Answer with the new requirement, don't explain.",
                },
                {
                    "role": "user",
                    "content": "Specification: "
                    + spec
                    + "(end of the specification) List of requirements: "
                    + req_list
                    + "(end of list of requirements).",
                },
            ],
            max_tokens=50,
        )
        res = chat_response.choices[0].message.content
        log(
            "I had an interaction with mistral to generate a requirement. "
            + "I gave the following spec: "
            + spec
            + "\n"
            + "I also gave the following requirements: "
            + req_list
            + "\n"
            + "Mistral gave me the following answer: "
            + res
        )
        return res
    except Exception as e:
        print(str(e))
        raise AslError("LLM failure: " + str(type(e))) from e


def prompt_completeness(
    s: agentspeak.Literal, r: agentspeak.Literal
) -> agentspeak.Literal:
    assert s.functor == "spec"
    assert r.functor == "req"
    try:
        res = ask_llm_for_coverage(str(s.args[0]), str(r.args[0]))
        return agentspeak.Literal("complete" if res else "incomplete")
    except Exception as e:
        print("Failure while talking with the llm")
        print(str(e))
        return failure


def prompt_generation(s, r) -> agentspeak.Literal:
    assert s.functor == "spec"
    assert r.functor == "req"
    try:
        res = ask_llm_for_completion(str(s.args[0]), str(r.args[0]))
        res = res if res.startswith('"') and res.endswith('"') else neutralize_str(res)
        return agentspeak.Literal(res)
    except Exception as e:
        print("Failure while talking with the llm")
        print(str(e))
        return failure


action_prompt_completeness = actions.Action(
    "function",
    ".prompt_completeness",
    (agentspeak.Literal, agentspeak.Literal),
    prompt_completeness,
)

action_prompt_generate = actions.Action(
    "function",
    ".prompt_generate",
    (agentspeak.Literal, agentspeak.Literal),
    prompt_generation,
)



from a2a_agentspeak.content_codecs import python_agentspeak_codec
from a2a_agentspeak.content_codecs.common import (
    cold_repository_up_codec_id,
    cold_repository_down_codec_id,
    Codec,
)
from cold_repository.codec import decode_cold_descr


class ColdRepositoryUpCodec(Codec):
    """Messages sent to a cold repository.
    Example : cold_by_skill(...)"""

    def __init__(self):
        super().__init__(cold_repository_up_codec_id)

    def extract_entry_point(self, s: str):
        if s.startswith("cold_by_skills("):
            return "cold_by_skills"
        else:
            print("Unrecognized entry point: " + s)
            raise NotImplementedError()

    def decode(self, s: str):
        if s.startswith("cold_by_skills("):
            return python_agentspeak_codec.codec_object.decode(
                s
            )  # fixme : remove dependency on python-agentspeak
        else:
            print("Unable to decode: " + s)
            raise NotImplementedError()


up_codec_object = ColdRepositoryUpCodec()


class ColdRepositoryDownCodec(Codec):
    """Messages sent by cold repository.
    Example: selected(cold(agent(...))"""

    def __init__(self):
        super().__init__(cold_repository_down_codec_id)

    def extract_entry_point(self, s: str):
        if s.startswith("selected("):
            return "selected"
        else:
            print("Unrecognized entry point: " + s)
            raise NotImplementedError()

    def decode(self, s: str):
        if s.startswith("selected(cold_agent("):
            s2 = s.removeprefix("selected(").removesuffix(")")
            return decode_cold_descr(s2)
        else:
            print("Unable to decode: " + s)
            raise NotImplementedError()


down_codec_object = ColdRepositoryDownCodec()



from a2a_acl.a2a_utils.send_message import extract_text_from_message, extract_text_from_task

import context
from agentspeak import LinkedList, Literal

from a2a.utils import new_agent_text_message

from a2a.server.events import EventQueue


from a2a_acl.agent.acl_agent import ACLAgentExecutor
from a2a_acl.agent.server_utils import run_server
from a2a_acl.protocol_extension.acl_message import ACLMessage, Illocution

from a2a_agentspeak.content_codecs import cold_repository_codec
from a2a_agentspeak.content_codecs.common import (
    python_agentspeak_codec_id,
    cold_repository_down_codec_id,
    cold_repository_up_codec_id, atom_codec_id,
)

from a2a_acl.interface.interface import SkillDeclaration, ACLAgentCard
from a2a_agentspeak.build_server import build_and_run
from a2a_acl.protocol_extension.send_acl_message import (
    spawn_send_acl_message
)

from a2a_acl.a2a_utils.card_holder import download_card
from cold_repository import codec
from a2a_acl.utils.url import build_url


# We first define an agent that will receive the pong messages from the spawned agent.

my_skill_1 = SkillDeclaration(Illocution.TELL, "pong", 0, "Receive a pong reply.")
my_skill_2 = SkillDeclaration(
    Illocution.TELL, "selected", 2, "Receive a cold agent."
)
my_skills = [my_skill_1, my_skill_2]

class ClientAgentExecutor(ACLAgentExecutor):

    def __init__(self, host, port):
        super().__init__(host,port)
        self.codec_objects[cold_repository_down_codec_id]= cold_repository_codec.down_codec_object

    async def execute_tell(
        self,
        m: ACLMessage,
        output_event_queue: EventQueue,
    ) -> None:
        res = m.content
        print("Incoming message: " + res)
        print("TEST " + ("OK" if res == "pong" else "KO"))
        await output_event_queue.enqueue_event(
            new_agent_text_message("MESSAGE RECEIVED")
        )

my_url = context.client_url


def decode_and_spawn(res:str):
        """ Decode a message from a cold repository,
            then spawn the agent described in that message,
            and send a achieve:ping message to that agent.
        """
        (filename, holes) = codec.decode_cold_descr(res)

        build_and_run(
            context.path_to_sample_agents + filename, context.host, context.free_port
        )

        spawn_send_acl_message(
            build_url(context.host, context.free_port),
            "achieve",
            "ping",
            my_url,
            atom_codec_id,
        )


async def main() -> None:

    # 1) start an a2a server to receive asynchronous answers
    my_card = ACLAgentCard(
        "Client Agent",
        "A client agent",
        my_skills,
        [cold_repository_down_codec_id, python_agentspeak_codec_id, atom_codec_id],
    )

    run_server(ClientAgentExecutor(my_card, my_url), context.host, context.client_port)

    # 2) query the repository

    try:
        repository_card = await download_card(context.repository_url)
    except Exception:
        print("Repository not available. Cannot continue.")
        exit(-1)

    test_skill = Literal("mandatory_skill", (Literal("achieve"), Literal("ping"), 0))
    lst = LinkedList(test_skill, [])
    request = str(Literal("cold_by_skills", (lst,)))

    print("Sending a request to the repository.")
    spawn_send_acl_message(
        repository_card, "ask", request, my_url,
        codec = cold_repository_up_codec_id,
        message_processor=lambda message: decode_and_spawn(extract_text_from_message(message)),
        task_processor=lambda message: decode_and_spawn(extract_text_from_task(message))
    )



if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



import os

import agentspeak
import litellm
from agentspeak import AslError


from a2a_agentspeak import actions
from a2a_acl.utils.strings import neutralize_str


llm_model = "gpt-4o-mini"
os.environ["LANGFUSE_OTEL_HOST"] = "https://cloud.langfuse.com"
litellm.callbacks = ["langfuse_otel"] # pip install litellm opentelemetry-api opentelemetry-sdk opentelemetry-exporter-otlp

llm_timeout = 60  # seconds


failure = agentspeak.Literal("failure")


def log(m):
    print("[LOG] " + m)


def ask_llm_for_coverage(spec: str, req_list: str) -> bool:
    log("Asking LLM for coverage (timeout " + str(llm_timeout) + " seconds)")
    try:
        chat_response = litellm.completion(
            model=llm_model,
            messages=[
                {   "role":"system",
                    "content":"Given a specification of a system, and a list of atomic requirements, tell if that list of atomic requirements covers well that specification."
                    + " Answer COMPLETE is the specification is well covered."
                    + " Answer PARTIAL otherwise."
                },
                {
                    "role":"user",
                    "content":"Specification: "
                    + spec
                    + "(end of the specification) List of requirements: "
                    + req_list
                    + "(end of list of requirements)."
                }],
            timeout=llm_timeout,
            max_tokens=16,  # 16 is the minimum for gpt-4o-mini
        )
        log("Response from LLM received")
        r = chat_response.choices[0].message.content
        log(
            "I had an interaction with gpt to check coverage.\n "
            + "I gave the following spec: "
            + spec
            + "\n"
            + "I also gave the following requirements: "
            + req_list
            + "\n"
            + "Gpt gave me the following answer: "
            + r
        )
        if r.startswith("COMPLETE"):
            return True
        elif r.startswith("PARTIAL"):
            return False
        else:
            raise Exception("Cannot understand LLM answer.")
    except RuntimeError as e:
        print("Runtime Error: " + str(e))
        raise AslError("LLM failure: " + str(type(e))) from e
    except Exception as e:
        print("Exception: " + str(e))
        raise AslError("LLM failure: " + str(type(e))) from e


def ask_llm_for_completion(spec: str, req_list: str):
    log("Asking LLM for completion  (timeout " + str(llm_timeout) + " seconds)")
    try:

        chat_response = litellm.completion(
            model=llm_model,
            messages=[{
                "role":"system",
                "content":"Given a specification of a system, and a list of atomic requirements, give an atomic requirements that covers the specification and which is not included in the given list of requirements."
                + " Answer with the new requirement, don't explain."
                },
                { "role":"user",
                  "content":"Specification: "
                    + spec
                    + "(end of the specification) List of requirements: "
                    + req_list
                    + "(end of list of requirements)."
                  }
            ],
            timeout=llm_timeout,
            # max_tokens=50,
        )
        log("Resonse from LLM received")
        res = chat_response.choices[0].message.content
        log(
            "I had an interaction with gpt to generate a requirement. "
            + "I gave the following spec: "
            + spec
            + "I also gave the following requirements: "
            + req_list
            + " "
            + "Gpt gave me the following answer: "
            + res
        )
        return res
    except Exception as e:
        print(str(e))
        raise AslError("LLM failure: " + str(type(e)))


def prompt_completeness(
    s: agentspeak.Literal, r: agentspeak.Literal
) -> agentspeak.Literal:
    assert s.functor == "spec"
    assert r.functor == "req"
    try:
        res = ask_llm_for_coverage(str(s.args[0]), str(r.args[0]))
        return agentspeak.Literal("complete" if res else "incomplete")
    except Exception as e:
        print("Failure while talking with the llm")
        print(str(e))
        return failure


def prompt_generation(s, r) -> agentspeak.Literal:
    assert s.functor == "spec"
    assert r.functor == "req"
    try:
        res = ask_llm_for_completion(str(s.args[0]), str(r.args[0]))
        res = res if res.startswith('"') and res.endswith('"') else neutralize_str(res)
        return agentspeak.Literal(res)
    except Exception as e:
        print("Failure while talking with the llm")
        print(str(e))
        return failure


action_prompt_completeness = actions.Action(
    "function",
    ".prompt_completeness",
    (agentspeak.Literal, agentspeak.Literal),
    prompt_completeness,
)

action_prompt_generation = actions.Action(
    "function",
    ".prompt_generate",
    (agentspeak.Literal, agentspeak.Literal),
    prompt_generation,
)



import time

import context
from a2a_agentspeak import build_server
from sample_agents.requirement_generators.atomic import mistral_requirement_actions

if __name__ == "__main__":

    host = "127.0.0.1"
    filename = "../../sample_agents/requirement_generators/atomic/llm_requirement_generator"

    build_server.build_and_run(
        filename,
        host,
        context.generator_port,
        [
            mistral_requirement_actions.action_prompt_generate,
        ],
    )

    # needed when using LiteLLM
    input("Press Enter to quit.\n")


import asyncio

from a2a.types import TaskStatusUpdateEvent, Task
from a2a_acl.protocol_extension.send_acl_message import spawn_send_acl_message, send_acl_message
from a2a_acl.utils.strings import neutralize_str
from a2a_acl.utils.url import build_url

import context
from a2a.server.events import EventQueue

from a2a_acl.agent.acl_agent import ACLAgentExecutor
from a2a_acl.content_codecs.common import natural_language_id

from a2a_acl.protocol_extension.acl_message import ACLMessage
from a2a_acl.a2a_utils.send_message import sync_reply, extract_text_from_message, extract_text_from_task_artifact

from a2a_acl.interface import asi_parser
from a2a_acl.interface.interface import ACLAgentCard

from a2a_agentspeak.content_codecs.common import atom_codec_id, python_agentspeak_codec_id

my_card = asi_parser.read_file("reference_agent.asi").add_codecs([natural_language_id])


def ref_status_update_callback(t:Task, u:TaskStatusUpdateEvent, m:str):
    print("Task:" + t.id)
    print("Received an update: " + str(u.status.state))
    if u.status.message is not None:
        if len(u.status.message.parts)>0 :
            print("Message: ", u.status.message.parts[0].root.text)

def ask_validation(t: str):
    print("Proposed requirement: " + t)
    print("Do you accept this proposal ?")
    res = input("[Y/N/Q]\n")
    match res:
        case "Y" | "y":
            print("Accepted. (" + res + ")")
            return ("tell","valid")
        case "Q" | "q":
            print("Quit. (" + res + ")")
            return ("achieve","quit")
        case _:
            print("Rejected. (" + res + ")")
            return ("tell","invalid")




class ReferenceAgentExecutor(ACLAgentExecutor):
    """An agent that receives validation requests and
    which delegates the decision to a human user."""

    def __init__(self, card: ACLAgentCard, url):
        super().__init__(agentcard=card, my_url=url)
        print(
            "Warning: launching a pure python A2A agent, unable to check .asi interface against python body."
        )
        self.reference_spec = context.spec

    async def execute_achieve(
        self,
        m: ACLMessage,
        output_event_queue: EventQueue,
    ):
        print("Received a request to launch the generation.")
        await self.init_interaction(self.reference_spec, self.my_url)

    async def execute_tell(
        self,
        m: ACLMessage,
        output_event_queue: EventQueue,
    ):
        self.reference_spec = m.content
        await sync_reply(output_event_queue, "ack")
        print("Received an update of the specification to manage.")

    def callback_req_to_validate (self, t:Task):
        r = extract_text_from_task_artifact(t)
        print("Artifact added with requirement to validate" + r)
        (illoc,res) = ask_validation(r)
        spawn_send_acl_message(build_url(context.the_host, context.manager_port), illocution=illoc, sender=self.my_url, content=res, codec=atom_codec_id, artifact_update_callback=self.callback_final_result)

    def callback_final_result (self, t:Task):
        r = extract_text_from_task_artifact(t)
        print("Artifact received: " + r)



    async def init_interaction(self, spec:str, self_url):
        await send_acl_message(
            build_url(context.the_host, context.manager_port),
            "tell",
            "spec(" + neutralize_str(spec) + ")",
            self_url,
            python_agentspeak_codec_id,
            message_callback=(
                lambda m: print("Message event: " + extract_text_from_message(m))
            ),
            artifact_update_callback=self.callback_req_to_validate,
            status_update_callback=lambda t, u, m: print("Received an update: " + str(u)),
        )
        print("[tell/spec message sent to the manager agent.]")

        await asyncio.sleep(1)

        await send_acl_message(
            build_url(context.the_host, context.manager_port),
            "achieve",
            "request",
            self_url,
            atom_codec_id,
            message_callback=(
                lambda m: print("Message event: " + extract_text_from_message(m))
            ),
            artifact_update_callback=self.callback_req_to_validate,
            status_update_callback=ref_status_update_callback,
        )
        print("[Achieve message sent to the manager agent.]")




import a2a_acl.utils.strings
from a2a_acl.content_codecs.common import natural_language_id

import context

from a2a.types import (
    AgentCard,
)

from a2a_acl.protocol_extension.send_acl_message import (
    spawn_send_acl_message,
)
from a2a_acl.a2a_utils.card_holder import download_card


from a2a_acl.utils.url import build_url

from a2a_agentspeak.content_codecs.common import python_agentspeak_codec_id

host = "127.0.0.1"
code_to_analyse = context.code


async def main() -> None:
    target_agent_url = build_url(host, context.manager_port)

    # Fetch Public Agent Card and Initialize Client
    target_agent_card: AgentCard | None = None

    try:
        target_agent_card = await download_card(target_agent_url)

    except Exception:
        print("Client failed to fetch the public agent card. Cannot continue.")
        exit(-1)

    spawn_send_acl_message(
        target_agent_card,
        "tell",
        "spec(" + a2a_acl.utils.strings.neutralize_str(context.specification) + ")",
        "",
        python_agentspeak_codec_id
    )

    print("[Tell-Message sent to advisor agent.]")

    await asyncio.sleep(1)

    spawn_send_acl_message(
        target_agent_card,
        "upload",
        code_to_analyse,
        "",
        natural_language_id
    )
    print("[Tell-Message sent to agent.]")


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())
    input("Press Enter to exit.\n")



import context

import threading

from a2a_agentspeak import build_server


host = "127.0.0.1"
port = context.port_receiver

name = "receiver"


if __name__ == "__main__":

    def start():
        build_server.build_and_run(name, host, port)

    threading.Thread(target=start).start()



import asyncio
import threading
import time

import uvicorn

from a2a_agentspeak import actions
from a2a_agentspeak.asp_build import from_files, ASPAgentBuilder
from a2a_acl.utils.url import build_url


def build_and_run(asp_filename: str, host:str, port:int, specific_actions=()):
    a: ASPAgentBuilder = from_files(
        asp_filename,
        actions=tuple(specific_actions)
        + (spawn_action,),  # forward reference / mutual recursion
    )

    # build and run the a2a server
    server = a.build_server(host, port)
    u_server = uvicorn.Server(uvicorn.Config(server.build(), host=host, port=port))

    def start_server():
        loop = asyncio.new_event_loop()  # loop = asyncio.get_event_loop()
        loop.run_until_complete(u_server.serve())
        loop.close()
        print("A2A Shutdown.")

    def start_asp_agent():
        a.executor.asl_agent.run()


    threading.Thread(target=start_server).start()
    print("Running a2a-server for " + asp_filename + " agent.")
    time.sleep(0.5)
    if not u_server.started:
        return False
    else:
        print("Starting agent behavior.")
        # running the internal behavior after opening the server
        threading.Thread(target=start_asp_agent).start()
        # asyncio.run(a.executor.asl_agent.run_async())
        return True


def do_spawn(fic: str, port=9990) -> str:
    host = "127.0.0.1"
    """A spawn that tries several ports until one available port is found (between 9990 and 9999)."""
    current_port = port
    success = False
    while current_port < 10000 and not success:
        success = build_and_run(fic, host, current_port)
        used_port = current_port
        current_port += 1
    if success:
        return build_url(host, used_port)
    else:
        raise Exception("Failed to spawn agent (no port available)")


spawn_action = actions.Action(
    "function",
    ".spawn",
    (str,),
    do_spawn,
)



from a2a_agentspeak import asp_message
from a2a_agentspeak.content_codecs.common import Codec, python_agentspeak_codec_id


class PythonAgentspeakCodec(Codec):

    def __init__(self):
        super().__init__(python_agentspeak_codec_id)

    def extract_entry_point(self, s: str):
        return self.decode(s).functor

    def decode(self, s: str):
        return asp_message.lit_of_str(s)


codec_object = PythonAgentspeakCodec()
